#lang racket

(require benchmark benchmark/types
         plot pict (only-in unstable/gui/slideshow tabular)
         racket/runtime-path)

(define-runtime-path here ".")

(define js-command (make-parameter #f)) ; ugly, but easy

(define (run-one filename+bench v*)
  (match-define `(,filename ,bench) filename+bench)
  (define output
    (with-output-to-string
      (lambda ()
        (parameterize ([current-directory here])
          ;; first version has no suffix
          (define v    (if (= v* 1) "" (number->string v*)))
          (define file (format "run-~a~a.js" filename v))
          (when (file-exists? file) ; different number of versions for each
            (system (format "~a ~a/~a"
                            (js-command) (path->string here) file)))))))
  (define matched
    (regexp-match (string-append bench "[^:]*: [0-9]+\n") output))
  (displayln ; benchmarking lib reads from stdout
   (cond [matched
          (define bench-time (first matched))
          (match-define (list _ time) (regexp-match ": ([0-9]+)\n$" bench-time))
          (string->number time)]
         [else ; version didn't exist
          0])))

(define (run-one-engine engine)
  (parameterize ([js-command engine])
    (run-benchmarks
     ;; filenames and benchmark names are slightly different. need both
     '(("richards"      "Richards")
       ("deltablue"     "DeltaBlue")
       ("raytrace"      "RayTrace")
       ("splay"         "Splay")
       ("navier-stokes" "NavierStokes")
       ("pdfjs"         "PdfJS")
       ("crypto"        "Crypto")
       ("box2d"         "Box2D"))
     '((1 2))
     run-one
     #:num-trials 30
     ;; run-one does the actual parsing
     #:extract-time (compose string->number string-trim)
     #:make-name second)))


;; Do we re-run the benchmarks (and/or re-render the pdfs) or just render the
;; picts for the paper?
(define re-run?         #f)
(define re-render-pdfs? #f)

;; A result-list is a (hashof <engine-name> benchmark-result?)

;; -> result-list
(define (run-all)
  (for/hash ([engine '("js" "v8" "jsc")])
    (define results-file (build-path here (string-append "results-" engine)))
    (define results
      (if re-run?
          (run-one-engine engine)
          (get-past-results results-file)))
    (when re-run?
      (record-results results results-file))
    (values engine results)))


(define (add-legend pict)
  (hc-append
   pict
   (blank 10)
   ;; legend
   (tabular
    (list (colorize (frame (filled-rectangle 8 8)
                           #:color "black" #:line-width 1)
                    "white")
          (text "Before" '() 7))
    (list (colorize (frame (filled-rectangle 8 8)
                           #:color "black" #:line-width 1)
                    "Dark Gray")
          (text "After" '() 7))
    #:gap 5 #:valign cc-superimpose)))

;; may emit a pdf plot as a side-effect
(define (plot-one engine results)
  (define y-label "Normalized score (higher is better)")
  (parameterize ([error-bar-width (if (equal? engine "js") 8 4)]
                 [plot-x-tick-label-angle (if (equal? engine "js") 0 30)]
                 [plot-x-tick-label-anchor (if (equal? engine "js") 'top 'top-right)])
    (define renderer
      (list (render-benchmark-alts
             '(1) ; normalize to first version
             results)
            (y-tick-lines)))
    (when re-render-pdfs?
      (plot-file renderer
                 (build-path here (format "~a-plot.pdf" engine))
                 #:x-label #f #:y-label y-label))
    ((if (equal? engine "js") add-legend values)
     (plot-pict renderer
                #:y-max 1.22
                #:x-label #f #:y-label (if (equal? engine "jsc") #f y-label)
                #:width (if (equal? engine "js") 380 210)
                #:height (if (equal? engine "js") 280 250)))))

;; result-list -> (hashof <engine-name> pict?)
(define (plot-all result-list)
  (for/hash ([(engine results) result-list])
    (values engine (plot-one engine results))))

;; plotting parameters
[plot-x-ticks no-ticks]
[plot-y-ticks (linear-ticks #:divisors '(5 2))]
[plot-y-far-ticks no-ticks]
[benchmark-show-legend? #f]
[current-benchmark-color-scheme black-white-color-scheme-short]
[error-bar-color "black"]
[error-bar-alpha 1]
[error-bar-line-width 1]
[rectangle-line-color "black"]
[plot-font-family 'swiss]
[plot-font-size 7]
;; TODO tweak those


(provide sm-plot v8-plot jsc-plot v8+jsc-plot)
(define all-plots (plot-all (run-all)))
(define sm-plot  (dict-ref all-plots "js"))
(define v8-plot  (dict-ref all-plots "v8"))
(define jsc-plot (dict-ref all-plots "jsc"))
(define v8+jsc-plot (add-legend (hbl-append 10 v8-plot jsc-plot)))
